import os
import pytest

import pandas as pd
from tests.funct.utils import (
    make_features_dataframe,
    make_labels_dataframe,
    features_root,
    labels_root,
    accuracy,
)

import legal_doc_processing as ldp
import legal_doc_processing.information_extraction as infext
import legal_doc_processing.segmentation as seg
import legal_doc_processing.utils as uts


class TestCase:
    """Test class for Case feature"""

    def test_case_accuracy(self, threshold: float = 0.90) -> None:
        """compute accuracy for case prediction; return None """

        # pipe
        nlpipe = infext.get_pipeline()

        # read df
        df = pd.read_csv("./data/csv/dataset.csv")
        df = df.iloc[:5, :]

        # X_test and y_test
        X_test = df.drop(
            "reference",
            axis=1,
        )
        y_test = df.reference.values

        # better y_test
        strip_lower = lambda i: str(i).strip().lower()
        y_test = [strip_lower(i) for i in y_test]

        # make pred
        predict = lambda txt: ldp.LegalDoc(txt, nlpipe=nlpipe).predict_case()
        y_pred = [strip_lower(predict(txt)) for txt in X_test.document_TEXT.values]

        test_vs_pred = list(zip(y_test, y_pred))

        # accuracy
        _1st_acc = accuracy(y_test, y_pred)
        # assert _1st_acc > threshold



from cleantext import clean
from legal_doc_processing import utils


def clean_doc(file_text):
    """ """

    pages = []
    for page in file_text.split("\x0c"):

        # clen text
        page_meta = [
            {
                "text": clean(
                    para,
                    lower=False,
                ).replace("_", "")
            }
            for para in page.split("\n")
        ]

        clean_page = []
        previous_line = {}
        text = ""

        # add meta data
        for line in page_meta:
            line["is_section_num"] = utils.is_section_num(line["text"])
            line["is_title"] = utils.is_title(line["text"])
            line["ends_with_ponc"] = utils.ends_with_ponc(line["text"])
            line["is_alpha"] = sum(c.isalpha() for c in line["text"])
            line["start_with_upper"] = utils.starts_with_upper(line["text"])
            # not relevant line
            if not line["is_alpha"]:
                continue
            if not utils.same_sentence(previous_line, line):
                if text:
                    clean_page.append(text)
                    text = ""
            previous_line = line
            text = " ".join([text, line["text"]])

        if len(clean_page):
            pages.append(clean_page)

    return pages



from tests import *
from legal_doc_processing import utils


class TestTitle(unittest.TestCase):

    def test_romans_numbers(self):
        self.assertTrue(utils.is_section_num("IV."))

    def test_not_number(self):
        self.assertFalse(utils.is_section_num("lp"))


class TestPunctuation(unittest.TestCase):

    def test_punc(self):
        self.assertTrue(utils.ends_with_ponc("is ending with punc."))

    def test_not_punc(self):
        self.assertFalse(utils.ends_with_ponc("is not ending with punc"))


import os
import pytest

from tests.funct.utils import (
    make_features_dataframe,
    make_labels_dataframe,
    features_root,
    labels_root,
    accuracy,
)

import legal_doc_processing as ldp
from legal_doc_processing.utils import make_dataframe


class TestDataFrame:
    def test_build_3(self):
        """ """

        make_dataframe(n=3)

    def test_build_20(self):
        """ """

        make_dataframe(n=20)


import os

import pandas as pd

import legal_doc_processing as ldp
import legal_doc_processing.information_extraction as infext
import legal_doc_processing.segmentation as seg
import legal_doc_processing.utils as utls
from legal_doc_processing.information_extraction import _ask_who_is

from notebooks.packages import *
from notebooks.utils import *
from notebooks.press_release import get_press_dummy_defendant, get_press_nlp_defendant


nlpipe = get_pipeline()

if __name__ == "__main__":

    # press_list df
    press_list = pd.DataFrame(
        [
            {"root": str(os.path.dirname(i)), "press_file": os.path.basename(i)}
            for i in x_data_files(50, "press")
        ]
    )
    press_list.set_index("root", inplace=True)

    # order_list df
    order_list = pd.DataFrame(
        [
            {"root": str(os.path.dirname(i)), "order_file": os.path.basename(i)}
            for i in x_data_files(50, "order")
        ]
    )
    order_list.set_index("root", inplace=True)

    # join in a df
    df = pd.concat(
        [order_list, press_list],
        join="inner",
        axis=1,
    )
    df["root"] = [str(i) for i in df.index.values]
    df.index = range(len(df))

    # preds labmda
    jawad_preds = lambda ser: (
        ldp.read_file(ser.root + "/" + ser.order_file, nlpipe=nlpipe)
    ).predict_defendant()

    press_dummy_pred = lambda ser: get_press_dummy_defendant(
        utls.load_data(ser.root + "/" + ser.press_file)
    )

    press_nlp_pred = lambda ser: get_press_nlp_defendant(
        utls.load_data(ser.root + "/" + ser.press_file), nlpipe=nlpipe
    )

    # make preds
    df["jawad_preds"] = df.apply(jawad_preds, axis=1)
    df["press_dummy_preds"] = df.apply(press_dummy_pred, axis=1)
    df["press_nlp_preds"] = df.apply(press_nlp_pred, axis=1)



# #!/usr/bin/env python
# # coding: utf-8

# # ## 1 - Impots
# # --------------------


# import legal_doc_processing as ldp
# from legal_doc_processing.information_extraction import *
# from legal_doc_processing.segmentation import *
# from legal_doc_processing.utils import *

# from notebooks.packages import *


# # ## Extracted authorities


# """
# UNITED STATES DISTRICT COURT
# FOR THE MIDDLE DISTRICT OF FLORIDA
# """

# # ## Code law violation


# """
# violated Section 4c(a)(5)(C) of the Commodity Exchange Act ("Act"), 7 U.S.C.
# § 6c(a)(5)(C) (2018).
# """


# """
# Section 4c(a)(5) of the Act, 7 U.S.C.
# § 6c(a)(5) (2012).
# """

# # ## Violation period


# line = 'The Commodity Futures Trading Commission ("Commission") has reason to believe that from at least July 2012 through March 2017 ("Relevant Period"), Propex Derivatives Pty Ltd. ("Propex") violated Section 4c(a)(5)(C) of the Commodity Exchange Act ("Act"), 7 U.S.C. 6c(a)(5)(C) (2018). Therefore, the Commission deems it appropriate and in the public interest that public administrative proceedings be, and hereby are, instituted to determine whether Propex engaged in the violations set forth herein and to determine whether any order should be issued imposing remedial sanctions.'


# ("Relevant Period")


# # ## Transaction amount


# """
# pour complaint tous les montants vers Transaction amounts
# """

# # ## Defendant


# """
# Propex Derivatives Pty Ltd, Respondent.
# """


# #!pip install -U spacy
# #!python -m spacy download en_core_web_sm


# # ## Nature of violations


# """
# II. FINDINGS
# The Commission finds the following:
# A. SUMMARY
# During the Relevant Period, Propex, by and through a Propex trader ("Trader A"),·
# engaged in thousands of instances of the disruptive trading practice known as "spoofing"
# (bidding or offering with the intent to cancel the bid or offer before execution) in the E-mini
# S&P 500 futures contracts traded on the Chicago Mercantile Exchange ("CME"), a futures
# exchange and designated contract market which is owned and operated by CME Group Inc. This
# conduct violated Section 4c(a)(5)(C) of the Act, 7 U.S.C. § 6c(a)(5)(C) (2018).
# """


# # cftc_full_list = pd.read_excel("cftc_full_list.xlsx")


# def gen_line_rep(rep):
#     original_path = f"./cftc/original/{rep}"
#     text_path = f"./cftc/text/{rep}"
#     meta_path = f"./cftc/meta-data/{rep}"
#     lines = []
#     for file in glob.glob(f"{meta_path}/*.json"):
#         filename = os.path.basename(file)

#         with open(file) as json_file:
#             data = json.load(json_file)

#         data["filename"] = filename.replace(".json", "")
#         data["folder"] = rep
#         with open(f"{text_path}/{filename.replace('.json','.txt')}") as f:
#             data["doc_text"] = f.read()

#         lines.append(data)
#     return lines


# meta_data = []
# for index, row in cftc_full_list[~cftc_full_list.scraped_folder.isnull()].iterrows():
#     try:
#         meta_data.extend(gen_line_rep(row["scraped_folder"]))
#     except:
#         print("error on", row["scraped_folder"])
# len(meta_data)


# df_meta_data = pd.DataFrame(meta_data)


# df_meta_data["doc_clean"] = df_meta_data.doc_text.apply(clean_doc)


# df_meta_data["first_page"] = df_meta_data["doc_clean"].str[0]


# df_meta_data = df_meta_data[~df_meta_data["first_page"].isnull()]


# # ## Get reference


# # df_meta_data["reference"] = df_meta_data.first_page.apply(get_case)


# # # ## Get defendant


# # first_page = [text for text in df_meta_data.first_page.values[3] if len(text) > 100]


# # first_page


# # print(nlp(question="Who violeted?", context=".".join(first_page)))


# # print(nlp(question="Who is the defendant?", context=".".join(first_page), topk=3))


# # df_meta_data[['reference', 'folder', 'filename']][df_meta_data.reference.isnull()]


# # ## Type


# # df_meta_data["is_order"] = df_meta_data.filename.str.contains("order")
# # df_meta_data["is_complaint"] = df_meta_data.filename.str.contains("complaint")


# # df_meta_data["type"] = np.where(
# #     df_meta_data["is_order"],
# #     "Order CFTC",
# #     np.where(df_meta_data["is_complaint"], "Complaint CFTC", None),
# # )



import os
import json

import pandas as pd


root_dir = "./tests/dataset/"
features_root = root_dir + "features/"
labels_root = root_dir + "labels/"


def make_features_dataframe(root=features_root):
    """using all the text files considers as feature, read, build and return a pd.DataFrame object """

    data_list = []
    for text in os.listdir(root):

        with open(root + "/" + text, "r") as f:
            data_list.append({"file_key": text, "text": f.read()})

    return pd.DataFrame(data_list)


def make_labels_dataframe(root=labels_root):
    """using all the json files considers as labels, read, build and return a pd.DataFrame object """

    data_list = []
    for text in os.listdir(root):

        with open(root + "/" + text, "r") as f:
            txt = f.read()
            data_list.append(json.loads(txt))

    return pd.DataFrame(data_list)


def accuracy(list_0: list, list_1: list) -> float:
    """compute basic accuracy score """

    if len(list_0) != len(list_1):
        raise AttributeError("list_0 and list_1 should be same length")

    if type(list_0[0]) != type(list_1[0]):
        raise AttributeError("list_0 and list_1 should be same type")

    return round((sum([tu[0] == tu[1] for tu in zip(list_0, list_1)]) / len(list_0)), 2)



import os

import legal_doc_processing.information_extraction as infext
import legal_doc_processing.segmentation as seg


class LegalDoc:
    """main legal doc class """

    def __init__(self, text: str, file_path: str = None, nlpipe=None):
        """init method of the LegalDoc
        pos args :
            text (str) : the complete text to proceed
            nlpipe (pipleline) : a pre instanciated pipline. default None (a object will be created)
        opt args :
            -
        raise :
            -
        return :
            a LegalDoc object"""

        # args as attr
        self.file_path = os.path.dirname(file_path) if file_path else None
        self.file_name = os.path.basename(file_path) if file_path else None
        self.nlpipe = nlpipe if nlpipe else infext.get_pipeline()

        # text and clean
        self.raw_text = text
        self.clean_pages = seg.clean_doc(text)

        # features
        self.case = None
        self.defendant = None

    def predict_case(self) -> str:
        """predict case, update self.case attr and return the value"""

        self.case = infext.get_case(self.clean_pages[0])

        return self.case

    def predict_defendant(self) -> str:
        """predict defendant, update self.defendant attr and return the value"""

        self.defendant = infext.get_defendant(self.clean_pages[0], self.nlpipe)

        return self.defendant

    def predict_all(self) -> dict:
        """perform various prediction, udate each attr and return a dict off attrs:value """

        pred = {}
        pred["case"] = self.predict_case()
        pred["defendant"] = self.predict_defendant()

        return pred

    def __repr__(self):
        """__repr__ method """

        return f"LegalDoc(path:{self.file_path}, file:{self.file_name}, case:{self.case}, defendant:{self.defendant}, pipe:{'OK' if self.nlpipe else self.nlpipe})"

    def __str__(self):
        """__str__ method """

        return "a LegalDoc Instance"


def read_file(file_path: str, nlpipe=None):
    """read a file and return a LegalDoc object """

    with open(file_path, "r") as f:
        text = f.read()

    ld = LegalDoc(text, file_path=file_path, nlpipe=nlpipe)
    return ld



# builtin
import os
import sys
import dateparser
import string
import re
import urllib.request
import json
import glob
import heapq

# request and web parse
import requests

# import bs4 as bs

# data
import pandas as pd
import numpy as np

# from sklearn.feature_extraction.text import CountVectorizer

# nlp
import nltk
from nltk.tokenize import sent_tokenize
from transformers import pipeline, AutoModelForTokenClassification, AutoTokenizer
from cleantext import clean
import spacy

# import textsplit
# from textsplit.tools import SimpleSentenceTokenizer
# from textsplit.tools import get_penalty, get_segments
# from textsplit.algorithm import split_optimal, split_greedy, get_total
import word2vec

# local packages
import legal_doc_processing as ldp
from legal_doc_processing.information_extraction import *
from legal_doc_processing.segmentation import *
from legal_doc_processing.utils import *

# bootstrap packages
# nltk.download("stopwords")
# nltk.download("popular")
# nlp = spacy.load("en_core_web_sm")
# get_ipython().run_line_magic("matplotlib", "inline")


import os

import pandas as pd

import legal_doc_processing as ldp
from legal_doc_processing.information_extraction import *
from legal_doc_processing.information_extraction import _ask_who_is
from legal_doc_processing.segmentation import *
from legal_doc_processing.utils import *

from notebooks.packages import *
from notebooks.utils import *


nlpipe = get_pipeline()


def get_press_dummy_defendant(raw_text):
    """extract defendant """

    # extract lines with 'against'
    lines = [
        i.replace("\u200b", "").replace("\t", " ") for i in raw_text.splitlines() if i
    ]
    cands = [i for i in lines[:300] if "against" in i.lower()]

    # sep pred
    preds = [
        k.lower()
        .split("against")[1]
        .split("for")[0]
        .replace("defendant", "")
        .replace("defendants", "")
        .strip()
        .capitalize()
        for k in cands
        if len(k) > 5
    ]

    if preds:
        return preds[0]

    return "-- None --"


def get_press_nlp_defendant(raw_text, nlpipe=None):

    ans = _ask_who_is(raw_text, "defendant", nlpipe=nlpipe)

    return ans[0].get("answer")


if __name__ == "__main__":

    # file_path
    file_path_list = x_data_files(30, "press")

    # read file
    raw_text_list = [load_data(file_path) for file_path in file_path_list]
    raw_text_list[0]

    # clean and first
    clean_pages_list = [clean_doc(raw_text) for raw_text in raw_text_list]
    first_page_list = [pages[0] for pages in clean_pages_list]
    first_page_list[0]
    joined_first_page_list = ["\n".join(l) for l in first_page_list]

    # pred defendant dummy
    pred_dummy_list = [get_press_dummy_defendant(txt) for txt in joined_first_page_list]
    pred_dummy_list

    # zip
    list(zip(file_path_list, pred_dummy_list))

    # pred defendant nlp
    pred_nlp_list = [
        get_press_nlp_defendant(txt, nlpipe=nlpipe) for txt in joined_first_page_list
    ]
    pred_nlp_list

    # zip
    list(zip(file_path_list, pred_nlp_list))

    # pandas
    pseudo_df = list(
        zip(
            map(os.path.basename, file_path_list),
            map(os.path.dirname, [i.replace("data/", "") for i in file_path_list]),
            pred_dummy_list,
            pred_nlp_list,
        )
    )

    df = pd.DataFrame(pseudo_df, columns=["basename", "dir", "pred_pummy", "pred_lnp"])
    df


"""This module implements NLP methods to handle legal documentation. From text cleaning to information extraction, it centralizes main functionalities that simplify the work for text mining for this category of documents"""
import nltk

nltk.download("stopwords")
nltk.download("popular")

from legal_doc_processing.legal_doc import LegalDoc, read_file
from legal_doc_processing.boot import boot


from tests import *
from legal_doc_processing import information_extraction


class TestInformationExtraction(unittest.TestCase):
    def test_docket_case_1(self):
        self.assertEqual(
            information_extraction.get_case(["CFTC Docket NO. SD 20-01"]), "SD 20-01"
        )
        self.assertEqual(
            information_extraction.get_case(["the case is 12132-CV-1211"]),
            "12132-CV-1211",
        )


import re
import pickle

import spacy
from transformers import pipeline, AutoModelForTokenClassification, AutoTokenizer


def get_pipeline():
    """ build and return a piplein"""

    return pipeline(
        "question-answering",
        model="distilbert-base-cased-distilled-squad",
        tokenizer="distilbert-base-cased",
    )


def save_pipeline():
    """init a pipeline and save in a pk """

    with open("./utils/pipe.pk", "wb") as f:
        nlpipe = get_pipeline()
        obj = pickle.dumps(nlpipe)
        f.write(obj)


def load_pipeline():
    """load apickled object """

    with open("./utils/pipe.pk", "rb") as f:
        cand = f.read()
    nlpipe = pickle.loads(cand)

    return nlpipe


def get_case(first_page, length_treshold=50):
    """parse the first page line by line, matching a
    regex pattern refering to case feature
    example 'NO.: 14-CV-81216'
    return the result"""

    # dump small char lines
    first_page = [i for i in first_page if len(i) < length_treshold]

    # format result
    format_result = (
        lambda i: i.group(0)
        .upper()
        .replace("NO.", "")
        .replace("NO", "")
        .replace(":", "")
        .strip()
    )

    # first search smthg with - 99-CV-99999 -
    p = re.compile("\d*-?CV-\d+.*")
    for line in first_page:
        result = p.search(line.upper())
        if result:
            return format_result(result)

    # first search smthg with - No.: -
    p = re.compile("NO[\.:]\s*.+")
    for line in first_page:
        result = p.search(line.upper())
        if result:
            return format_result(result)

    return "-- error : case not founded --"


def _ask_who_is(txt: str, who: str, nlpipe=None) -> list:
    """ init a nlpipe if needed and ask who is the who """

    # init pipleine if needed
    if not nlpipe:
        nlpipe = get_pipeline()

    # pipe and return
    return nlpipe(question=f"Who is the {who}?", context=txt, topk=3)


def get_defendant(first_page: list, nlpipe=None) -> str:
    """from a list of text lines, create a pipelie if needed and asqk question """

    # first_page_100 = [text for text in first_page if len(text) > 100]
    joined_first_page = "\n".join(first_page)

    # ask
    ans = _ask_who_is(joined_first_page, "defendant", nlpipe)

    return ans[0]["answer"]



import legal_doc_processing as ldp
from legal_doc_processing.information_extraction import *
from legal_doc_processing.segmentation import *
from legal_doc_processing.utils import *

from notebooks.packages import *
from notebooks.utils import *


if __name__ == "__main__":

    # file_path
    file_path_list = x_data_files(20, "order")

    # read file
    raw_text_list = [load_data(file_path) for file_path in file_path_list]
    raw_text_list[0]

    # clean and first
    clean_pages_list = [clean_doc(raw_text) for raw_text in raw_text_list]
    first_page_list = [pages[0] for pages in clean_pages_list]
    first_page_list[0]

    # pred case
    case_list = [get_case(fp) for fp in first_page_list]
    case_list

    # zip
    ziped = list(zip(file_path_list, case_list))



import os
import pytest

import pandas as pd

from tests.funct.utils import (
    make_features_dataframe,
    make_labels_dataframe,
    features_root,
    labels_root,
    accuracy,
)
import legal_doc_processing as ldp
import legal_doc_processing.information_extraction as infext
import legal_doc_processing.segmentation as seg
import legal_doc_processing.utils as uts


class TestLegalDoc:
    """test class for the LegalDoc object """

    def test_legal_doc_basics(self):
        """test init and basic method of a legal doc oject"""

        # # file path
        # file_name = os.listdir(features_root)[0]
        # file_path = features_root + file_name

        # # init object
        # ld = ldp.read_file(file_path)

        df = pd.read_csv("./data/csv/dataset.csv")
        txt = df.loc[0, "document_TEXT"]
        ld = ldp.LegalDoc(txt)

        # case
        assert not ld.case
        assert ld.predict_case()
        assert ld.case
        assert isinstance(ld.case, str)

        # defendant
        assert not ld.defendant
        assert ld.predict_defendant()
        assert ld.defendant
        assert isinstance(ld.defendant, str)

        # predict all
        pred = ld.predict_all()
        assert pred
        assert isinstance(pred, dict)
        assert ("case" in pred.keys()) and ("defendant" in pred.keys())



import asyncio

import requests

import pandas as pd
import numpy as np

import re
import heapq

import nltk

from cleantext import clean

# nltk.download("stopwords")
stopwords = nltk.corpus.stopwords.words("english")


def load_data(file_path: str) -> str:
    """from file_path open read and return text; return text """

    if ".pdf" in file_path:
        raise AttributeError("Error : file recieved is a pdf, only txt supported")

    with open(file_path, "r") as f:
        txt = f.read()

    return txt


def clean_spec_chars(text: str) -> tuple:
    """first text cleaning based on regex, just keep text not spec chars
    return tupple of text"""

    # article text
    article_text = re.sub(r"\[[0-9]*\]", " ", text)
    article_text = re.sub(r"\s+", " ", article_text)

    # formated text
    formatted_article_text = re.sub("[^a-zA-Z]", " ", article_text)
    formatted_article_text = re.sub(r"\s+", " ", formatted_article_text)

    return article_text, formatted_article_text


# def handle_encoding(text: str) -> str:
#     """handle encoding problems and force ascii conversion ; return clean text """

#     # encoding the text to ASCII format
#     text_encode = text.encode(encoding="ascii", errors="ignore")

#     # decoding the text
#     text_decode = text_encode.decode()

#     # cleaning the text to remove extra whitespace
#     clean_text = " ".join([word for word in text_decode.split()])

#     return clean_text


def is_section_num(text: str) -> bool:
    """detect if a paragrath start with Roman letter ie is a section; return bool """

    return re.search(r"^[MDCLXVIAB]+\.$", text) is not None


def ends_with_ponc(text: str, punctuation: str = "!.?") -> bool:
    """detect if a text finsih with punctation end of sentence; return bool"""

    return text[-1] in punctuation if text else False


def starts_with_upper(text: str) -> bool:
    """fist letter of a text is upper; return bool"""

    return re.match("^[A-Z]", text) is not None


def is_title(text: str, threshold=0.6) -> bool:
    """detect if text is a title if threshold % of the letters are upper; return bool """

    uppers = [word.isupper() for word in text.split(" ")]

    return sum(uppers) / len(uppers) > threshold


def same_sentence(sent1, sent2):
    # print(sent1,"\n",sent2)

    # sent empty
    if (not sent1) or (not sent1["text"]):
        return True
    # section number
    if sent1["is_section_num"]:
        return True

    # very short sentence
    if len(sent1["text"]) < 50:
        return False

    # ponctuation
    if sent1["ends_with_ponc"]:
        return False

    if sent1["is_title"]:
        return False

    if sent2["is_title"]:
        return False

    return True


def get_token(text):
    sentence_list = nltk.sent_tokenize(text)
    sentence_list
    return sentence_list


def get_para(sentence_list):
    j = 0
    i = 0
    para = []
    paragraphes = {}
    idx = 0
    while i < len(sentence_list):
        para = []
        paragraphes["hearder"] = sentence_list[i]

        while not is_section_num(sentence_list[i]):
            para.append(sentence_list[i])
            i += 1

        if is_section_num(sentence_list[i - 1]):
            paragraphes["content"] = para
            paragraphes["id"] = idx
            idx = idx + 1

    return paragraphes


def get_section_indx(list_token):

    idx = []
    for i in range(len(list_token)):
        if is_title(list_token[i]):
            idx.append(i)

    return idx


def get_structure(text):
    list_token = get_token(text)
    idx = get_section_indx(list_token)
    j = 0

    structure = []
    k = 0
    for i in idx:
        section = {}
        section["content"] = " ".join(list_token[j:i])
        section["header"] = list_token[j]
        section["id"] = k
        structure.append(section)
        j = i
        k = k + 1

    return structure


def clean_doc(
    file_text,
):
    """ """

    pages = []
    for page in file_text.split("\x0c"):

        # clen text
        page_meta = [{"text": clean(para)} for para in page.split("\n")]
        clean_page = []
        previous_line = {}
        text = ""

        # add meta data
        for line in page_meta:
            line["is_section_num"] = is_section_num(str(line["text"]))
            line["is_title"] = is_title(str(line["text"]))
            line["ends_with_ponc"] = ends_with_ponc(str(line["text"]))
            line["is_alpha"] = sum(c.isalpha() for c in str(line["text"]))
            line["start_with_upper"] = starts_with_upper(str(line["text"]))

            # not relevant line
            if not line["is_alpha"]:
                continue

            if not same_sentence(previous_line, line):
                if text:
                    clean_page.append(text)
                    text = ""

            previous_line = line
            text = " ".join([text, str(line["text"])])

        if len(clean_page):
            pages.append(clean_page)

    return pages


def get_structured_document(file):
    text_structured = []
    file_cleaned = clean_doc(file)
    i = 0
    text_ = {}
    continu = []
    sec_num = 0
    while sec_num < len(file_cleaned):
        if is_title(file_cleaned[sec_num][0]):
            continu.append(file_cleaned[sec_num])
            text_["hearder"] = file_cleaned[sec_num][0]
            sec_num += 1
            while not is_title(file_cleaned[sec_num][0]) and sec_num < (
                len(file_cleaned)
            ):
                continu.append(file_cleaned[sec_num])
                sec_num += 1
            if is_title(file_cleaned[sec_num][0]):
                text_["content"] = continu
                text_["id"] = i
                text_["hearder"] = file_cleaned[sec_num][0]
                sec_num += 1
                continu = []
                i += 1
            else:
                text_["content"] = continu
                text_["id"] = i

        text_structured.append(text_)

    return text_structured


def word_frequency(text):
    stopwords = nltk.corpus.stopwords.words("english")
    word_frequencies = {}
    for word in nltk.word_tokenize(text):
        if word not in stopwords:
            if word not in word_frequencies.keys():
                word_frequencies[word] = 1
            else:
                word_frequencies[word] += 1
    return word_frequencies


def sentence_score(sentence_list, word_frequencies):
    sentence_scores = {}
    for sent in sentence_list:
        for word in nltk.word_tokenize(sent.lower()):
            if word in word_frequencies.keys():
                if len(sent.split(" ")) < 30:
                    if sent not in sentence_scores.keys():
                        sentence_scores[sent] = word_frequencies[word]
                    else:
                        sentence_scores[sent] += word_frequencies[word]
    return sentence_scores


def make_dataframe(
    path: str = "./data/csv/original_dataset.csv", n: int = 10
) -> pd.DataFrame:
    """on basis of csv dataframe with all features, data clean, scrap googleapi and insert text in the dataframe
    :param path  = the path to read original dataset
    :param n     = the n-st line to scrap, other will be droped
    :return      = a dataframe with original data cleaned + text of main doc and press release
    """

    # read df
    df = pd.read_csv(path)

    # keep cols
    keep_cols = [
        "id",
        "name",
        "status",
        "reference",
        "document_link",
        "press_release_link",
        "monetary_sanction",
        "currency",
        "type",
        "justice_type",
        "defendant",
        "decision_date",
        "extracted_violations",
    ]

    drop_cols = [i for i in df.columns if i not in keep_cols]
    df = df.drop(drop_cols, axis=1)

    # fill rate
    fill_rate = lambda col: (len(df) - sum(df[col].isna())) / len(df)
    df_rate_fill = [(col, round(fill_rate(col), 2)) for col in df.columns]

    # press_release and document_link
    for col, ext in [("press_release", ".html"), ("document", ".txt")]:
        funct = (
            lambda i: np.nan if ("storage.google" not in i) else i.replace(".pdf", ext)
        )
        df[col + "_URL"] = df[col + "_link"].apply(lambda i: funct(str(i).strip()))

    # clean lines without press or document
    df = df.loc[~df.document_URL.isna(), :]
    df = df.loc[~df.press_release_URL.isna(), :]
    df.index = range(len(df))

    def scrap(url: str):
        """ """

        try:
            print(url)
            res = requests.get(url)

            if res.status_code < 300:
                return res.text
            else:
                return res.status_code

        except Exception as e:
            return str(e)

    # test on 10
    df = df.iloc[:n, :]

    # sync version
    for col in ["press_release", "document"]:
        df[col + "_TEXT"] = df[col + "_URL"].apply(lambda i: scrap(str(i).strip()))

    df.to_csv("./data/csv/dataset.csv", index=False)

    return df


import legal_doc_processing as ldp
from legal_doc_processing.information_extraction import *
from legal_doc_processing.segmentation import *
from legal_doc_processing.utils import *

from notebooks.packages import *
from notebooks.utils import *


# pipe
nlpipe = pipeline(
    "question-answering",
    model="distilbert-base-cased-distilled-squad",
    tokenizer="distilbert-base-cased",
)
if __name__ == "__main__":

    # file_path
    file_path_list = x_data_files(30, "order")

    # read file
    raw_text_list = [load_data(file_path) for file_path in file_path_list]
    raw_text_list[0]

    # clean and first
    clean_pages_list = [clean_doc(raw_text) for raw_text in raw_text_list]
    first_page_list = [pages[0] for pages in clean_pages_list]
    first_page_list[0]

    # clean first join
    joined_first_page_list = ["\n".join(first_page) for first_page in first_page_list]

    # # violated
    # violeted_ans_list = [
    #     nlpipe(question="Who violeted?", context=jfp, topk=3)
    #     for jfp in joined_first_page_list
    # ]
    # violeted_ans_list[:1]

    # violated_ans_peds = [preds[0].get("answer") for preds in violeted_ans_list]

    # defendant
    defendant_ans_list = [
        nlpipe(question="Who is the defendant?", context=jfp, topk=3)
        for jfp in joined_first_page_list
    ]
    defendant_ans_list[:1]

    defendant_ans_preds = [preds[0].get("answer") for preds in defendant_ans_list]

    # zip
    ziped = list(zip(file_path_list, defendant_ans_preds))



import setuptools
from legal_doc_processing.version import Version

REQUIRED = [
    "pandas",
    "nltk",
    "transformers",
    "spacy",
    "tensorflow",
    "clean-text",
    "word2vec",
]

setuptools.setup(
    name="legal_doc_processing",
    version=Version("1.0.2").number,
    description="Theolex document processing",
    long_description=open("README.md").read().strip(),
    long_description_content_type="text/x-rst",
    author="Jawad Alaoui",
    author_email="jawad@theolex.io",
    url="http://theolex-document-processing",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=REQUIRED,
    license="MIT License",
    zip_safe=False,
    keywords="theolex document processing package",
    #  classifiers=['Packages', 'NLP']
)



from datetime import date

from tests import *
from tests.helpers import *


class TestCleanDoc(unittest.TestCase):

    def test_helper(self):
        self.assertEqual(MyHelper.days_ago(date.today()), 0)

    def test_other_helper(self):
        assert tests_helper.list_has(3, [1, 2, 3])

    @pytest.mark.xfail
    def test_should_fail(self):
        self.assertEqual(False, True)



import os


def clean_data_folder(root="data/files/"):
    """read all data/*/*, delete .pdf, keep only .txt """

    folders = os.listdir(f"{root}")
    all_files = list()

    _ = [
        all_files.extend([f"{root}{fold}/{fi}" for fi in os.listdir(f"{root}{fold}")])
        for fold in folders
    ]

    # files
    not_text = [i for i in all_files if ".txt" not in i]
    only_text = [i for i in all_files if ".txt" in i]

    # remove not .txt
    _ = [os.remove(i) for i in not_text]


def find_files_type(root="data/files/"):
    """ """

    # all files
    folders = os.listdir(f"root")
    all_files = list()

    _ = [
        all_files.extend([f"{root}/{fold}/{fi}" for fi in os.listdir(f"{root}/{fold}")])
        for fold in folders
    ]

    # files text not text
    not_text = [i for i in all_files if ".txt" not in i]
    only_text = [i for i in all_files if ".txt" in i]

    # press / not press
    text_not_press = [i for i in only_text if "press" not in i]
    text_only_press = [i for i in only_text if "press" in i]

    # files
    files = sorted(["-".join(fn.split("/")[-1].split("-")[:2]) for fn in text_not_press])

    # ans is
    files_types = [
        "amended-complaint",
        "cftc-whistleblower",
        "complaint",
        "commissioner",
        "consent-final",
        "concurring-statement",
        "consent-order",
        "contempt-order",
        "default-judgment",
        "enforcement-advisory",
        "final-judgment",
        "final-order",
        "judgment",
        "memorandum",
        "non-prosecution",
        "notice",
        "opinion-and",
        "opinion-order",
        "order",
        "proposed-consent",
        "remarks-of",
        "report-and",
        "statutory-restraining",
        "supplement-order",
        "supplemental-consent",
    ]


def x_data_files(num: int = 10, _type: str = "order", root="data/files/") -> list:
    """find in the -num- st data folders the files with good type (ie order or press) make a filter
    and retunr the list of files with usable paths"""

    # sanity check
    assert _type in ["press", "order", "complaint"]

    # list of data/*
    x_file_paths = os.listdir(root)[:num]

    # for each path, list dir and filter
    x_files_list = list()
    _ = [
        x_files_list.extend(
            [
                root + path_ + "/" + i
                for i in os.listdir(root + path_)
                if ((_type in i) and (".pdf" not in i))
            ]
        )
        for path_ in x_file_paths
    ]

    return x_files_list


import asyncio

import requests

import pandas as pd
import numpy as np


# from


# async def count():
#     print("One")
#     await asyncio.sleep(1)
#     print("Two")

# async def main():
#     await asyncio.gather(count(), count(), count())


# L = list()


# # async
# async def async_funct(url: str):
#     """ """

#     try:
#         print(url)
#         res = await requests.get(url)

#         if res.status_code < 300:
#             L.append((url, res.text))
#         else:
#             L.append((url, res.status_code))
#         return None

#     except Exception as e:
#         L.append((url, str(e)))
#         return None


# async def bulk_crawl(df, col):
#     """ """

#     tasks = [async_funct(url) for url in df[col].values]

#     await asyncio.gather(*tasks)


# # run

# asyncio.run(bulk_crawl(_df, "press_release_URL"))



import pytest

from tests.funct.utils import (
    make_features_dataframe,
    make_labels_dataframe,
    features_root,
    labels_root,
    accuracy,
)


class TestAccuracy:
    """test accuracy is good """

    def test_basic_accuracy_pass(self):
        """test basic configuration where accuracy is ok """

        l_0 = ["a", "b", "c"]
        l_1 = ["a", "b", "c"]

        assert accuracy(l_0, l_1) > 0.99

        l_0 = ["a", "b", "ce"]
        l_1 = ["a", "b", "c"]

        assert accuracy(l_0, l_1) > 0.66

        l_0 = ["a", "bi", "ce"]
        l_1 = ["a", "b", "c"]

        assert accuracy(l_0, l_1) > 0.32

        l_0 = ["ap", "bi", "ce"]
        l_1 = ["a", "b", "c"]

        assert accuracy(l_0, l_1) < 0.01

    def test_basic_accuracy_fail(self):
        """test basic configuration where test fail"""

        l_0 = [1, "b", "ce"]
        l_1 = ["a", "b", "c"]

        with pytest.raises(AttributeError):
            accuracy(l_0, l_1)

        l_0 = ["a", "b"]
        l_1 = ["a", "b", "c"]

        with pytest.raises(AttributeError):
            accuracy(l_0, l_1)
